#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include "iolib.h"
const char *progname = "anonidentd";
const char *proghelp =
	"Usage: anonidentd [OPTIONS]\n"
	"A simple anonymous identd server.\n\n"
	"-n        Return a HIDDEN-USER response\n"
	"-c ID     Return a USERID:UNIX ID response\n";
#include "freedt.h"

char *response = NULL;

int ungot = -1;
int char_count = 0;

char get_char() {
	char c;
	if (ungot != -1) {
		c = ungot;
		ungot = -1;
	} else {
		if (iread(ibin, &c, 1) < 1)
			die("error reading from client");
		if (++char_count > 1000)
			die("client request too large");
	}
	return c;
}

void unget_char(char c) {
	ungot = c;
}

char get_char_space() {
	char c;
	do {
		c = get_char();
	} while (c == ' ' || c == '\t');
	return c;
}

int is_digit(char c) {
	return (c >= '0' && c <= '9');
}

int get_integer() {
	int n = 0, count = 0;
	char c = get_char_space();
	if (!is_digit(c))
		die("expected integer");
	do {
		if (++count > 5)
			die("integer too long");
		n = (n * 10) + (c - '0');
		c = get_char();
	} while (is_digit(c));
	unget_char(c);
	return n;
}

void get_comma() {
	if (get_char_space() != ',')
		die("expected comma");
}

void get_crlf() {
	char c, lc;
	c = get_char_space();
	while (1) {
		lc = c;
		c = get_char();
		if (c == '\n' && lc == '\r')
			return;
		if (c != ' ' && c != '\t' && c != '\r')
			die("expected CRLF");
	}
}

int main(int argc, char **argv) {
	int my_port, their_port;
	char *remoteip, *remoteport;

	istdinit();

	while (1) {
		int c = getopt(argc, argv, "V?nc:");
		if (c == -1)
			break;
		switch (c) {
		case 'n':
			response = NULL;
			break;
		case 'c':
			response = strdup(optarg);
			break;
		case 'V':
			version();
		default:
			help();
		}
	}

	if (argc != optind)
		help();

	setuidgidroot();

	remoteip = getenv("TCPREMOTEIP");
	if (remoteip == NULL)
		die("no TCPREMOTEIP set");
	remoteport = getenv("TCPREMOTEPORT");
	if (remoteport == NULL)
		die("no TCPREMOTEPORT set");

	my_port = get_integer();
	get_comma();
	their_port = get_integer();
	get_crlf();

	iformat(iberr, 'c', "Request from ", 'c', remoteip, 'c', " port ",
		'c', remoteport, 'c', " for ", 'i', my_port, 'c', ",",
		'i', their_port, 'c', "\n", 0);
	iflush(iberr);

	if (my_port < 1 || my_port > 65535)
		die("local port out of range");
	if (their_port < 1 || their_port > 65535)
		die("remote port out of range");

	if (iformat(ibout, 'c', " ", 'i', my_port, 'c', " , ",
			'i', their_port, 0) < 0)
		die("error writing to client");

	if (response == NULL) {
		if (iformat(ibout, 'c', " : ERROR : HIDDEN-USER\r\n", 0) < 0)
			die("error writing to client");
	} else {
		if (iformat(ibout, 'c', " : USERID : UNIX :", 'c', response,
				'c', "\r\n", 0) < 0)
			die("error writing to client");
	}

	iflush(ibout);

	return 0;
}

